-------------------------------------------------------------------------------
-- Title				:	Keypad controller
-- Project			:	Sample code
-- File				:	keypad_controller.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic PVt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	Displays the value of key pressed on the 7-segment display
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
-------------------------------------------------------------------------------
entity keypad_controller is
generic(rst_state:std_logic:='1');
    port (
		clk			: in  std_logic; 	    -- clock
        Rst         : in  std_logic; 	    -- System Reset       
        RL          : in  std_logic_vector(3 downto 0);		-- return lines
        SL          : out std_logic_vector(3 downto 0);		-- scan lines
        hit         : out std_logic;						-- key hit signal
        en		    : out std_logic_vector(1 downto 0);		-- 7-seg enable
        seg		    : out std_logic_vector(6 downto 0) 		-- key code in 7-segments
    );
end keypad_controller;
-------------------------------------------------------------------------------
architecture keypad_controller_arch of keypad_controller is
-------------------------------------------------------------------------------
signal Scan_Clk	: std_logic; 	--scan clock
signal Deb_Clk	: std_logic;  	-- debounce clock          	    
signal SL_s          :  std_logic_vector(3 downto 0);
signal or_key        :  std_logic;
signal D		     :  std_logic_vector(5 downto 0);
signal hit_s,deb_in  :  std_logic;
signal stop          :  std_logic;
signal key_code_s    :  std_logic_vector(7 downto 0);
signal div 		     : std_logic_vector(14 downto 0);
signal en_s 	     : std_logic;
signal release       :  std_logic;
signal key_LSB_s     : std_logic_vector(6 downto 0);
signal key_MSB_s     : std_logic_vector(6 downto 0);
signal en_clk        : std_logic;
begin

Scan_Clk  <= div(14) when stop='0' else
				 '0';

Deb_Clk  <= div(8);
en_clk   <= div(10);

en(0)    <= en_s;
en(1)    <= not en_s;
hit 		<= hit_s;
SL  		<= sl_s;

seg <= key_MSB_s    when en_s='0' else
       key_LSB_s   ;

-- stopping the scanning process when key pressed
	process(hit_s,release,Rst)
	begin
		if(Rst = rst_state) or release='1' then
			stop	<= '0';
		elsif(hit_s='1') then
			stop	<= '1';
		end if;
	end process;
    
-- Clock divider
    process(clk,rst)
    begin
    	if rst=rst_state then 
    		div	<= (others=>'0');
    	 elsif clk'event and clk='1' then
        	div <= div +1;
        end if ;
    end process;

-- generating enable for 7-segment disp
    process(en_clk,rst)
    begin
    	if rst=rst_state then 
    		en_s	<= '0';
    	 elsif en_clk'event and en_clk='1' then
        	en_s <= not en_s;
        end if ;
    end process;
-------------------------------------------------------------------------------
--	SCAN PATTERN GENERATOR
	process(Scan_Clk,Rst)
	begin
		if(Rst = rst_state)then
			sl_s <= "1110";
		elsif(Scan_Clk'event and Scan_Clk = '0')then
    			sl_s <= SL_s(0) & sl_s(3 downto 1);
		end if;
	end process;
-------------------------------------------------------------------------------          
-- process for key press detection
deb_in <= '1' when RL(0)='0' or RL(1)='0' or RL(2)='0' or RL(3)='0' else
          '0';
-------------------------------------------------------------------------------
--	DEBOUNCING THE 'OR' VALUE FOR 6 CLOCK CYCLES
	process(Deb_Clk,Rst)
	begin
		if(Rst = rst_state)then
			D <= (others => '0');
		elsif(Deb_Clk'event and Deb_Clk = '0')then
			D <= deb_in & D(5 downto 1);
	    end if;
	end process;
-------------------------------------------------------------------------------            
-- GENERATING INTERRUPT IF FOR 6 CLOCKS THE KEY IS PRESSED
-- Valid key hit
hit_s <= D(0) and D(1) and D(2) and D(3) and D(4) and D(5);

-- key released
release	<= not hit_s;      
-------------------------------------------------------------------------------
--	FOR LATCHING THE VALID KEY PRESSED
	process(hit_s,RL,Rst )
	begin
		if Rst =rst_state then
			key_code_s <= (others=>'0');
		elsif( hit_s = '1'and hit_s'event )then
			key_code_s <= sl_s & RL;
        end if;
	end process;
-------------------------------------------------------------------------------
-- Changing the key values in 7-segment coding.
with key_code_s select
--          abcdefg
key_LSB_s <="1111110" when "11101110",   -- 0
           	"0110000" when "11101101",   -- 1    
           	"1101101" when "11101011",   -- 2
            "1111001" when "11100111",   -- 3
            "0110011" when "11011110",   -- 4
            "1011011" when "11011101",   -- 5
            "1011111" when "11011011",   -- 6
            "1110000" when "11010111",   -- 7
            "1111111" when "10111110",   -- 8
            "1111011" when "10111101",   -- 9
            "0110000" when "10111011",   -- F1
            "1101101" when "10110111",   -- F2
            "1111001" when "01111110",   -- F3
            "0110011" when "01111101",   -- F4
            "1011011" when "01111011",   -- F5
            "1011111" when "01110111",   -- F6
            "0000001" when others;


with key_code_s select
--          abcdefg
key_MSB_s <="1111110" when "11101110",   -- 0
           "1111110" when "11101101",   -- 1    
           "1111110" when "11101011",   -- 2
           "1111110" when "11100111",   -- 3
           "1111110" when "11011110",   -- 4
           "1111110" when "11011101",   -- 5
           "1111110" when "11011011",   -- 6
           "1111110" when "11010111",   -- 7
           "1111110" when "10111110",   -- 8
           "1111110" when "10111101",   -- 9
           "1000111" when "10111011",   -- F1
           "1000111" when "10110111",   -- F2
           "1000111" when "01111110",   -- F3
           "1000111" when "01111101",   -- F4
           "1000111" when "01111011",   -- F5
           "1000111" when "01110111",   -- F6
	        "0000001" when others;

end keypad_controller_arch;
-------------------------------------------------------------------------------